from abc import ABC, abstractmethod
from mcapi.models import (
    PlainString,
    PlainInt,
    AllInfo,
    Players,
    BoolSchema,
    TupleSchema,
)
from mcaio.client import AIOMCServer


class IMC(ABC):
    """
    Async interface for interacting with Minecraft server
    """

    @property
    @abstractmethod
    async def name(self) -> PlainString:
        """
        Returns Minecraft server name
        :return: Minecraft server name
        """
        raise NotImplementedError

    @property
    @abstractmethod
    async def motd(self) -> PlainString:
        """
        Returns Minecraft server motd
        :return: Minecraft server motd
        """
        raise NotImplementedError

    @property
    @abstractmethod
    async def max_players(self) -> PlainInt:
        """
        Returns Minecraft server max players count
        :return: Minecraft server max players count
        """
        raise NotImplementedError

    @property
    @abstractmethod
    async def players_count(self) -> PlainInt:
        """
        Returns count of current online players
        :return: Count of current online players
        """
        raise NotImplementedError

    @property
    @abstractmethod
    async def players(self) -> TupleSchema:
        """
        Returns Minecraft server online players list
        :return: Minecraft server online players list
        """
        raise NotImplementedError

    @property
    @abstractmethod
    async def all(self) -> AllInfo:
        """
        Returns Minecraft server all available information
        :return: Minecraft server all available information
        """
        raise NotImplementedError

    @property
    @abstractmethod
    async def status(self) -> BoolSchema:
        """
        Returns Minecraft server online status
        :return: Minecraft server online status
        """
        raise NotImplementedError


class AIOMC(IMC):
    """
    mcaio-lib implementation for api
    """

    def __init__(self, host: str, port: int) -> None:
        """
        :param host: Minecraft server ip or hostname
        :param port: Minecraft server port
        """
        self.mc = AIOMCServer(host, port)

    @property
    async def name(self):
        return PlainString(data=await self.mc.name)

    @property
    async def motd(self):
        return PlainString(data=await self.mc.motd)

    @property
    async def max_players(self):
        return PlainInt(data=await self.mc.maxplayers)

    @property
    async def players_count(self):
        return PlainInt(data=await self.mc.players_count)

    @property
    async def players(self):
        plist = tuple(await self.mc.players_list)
        return TupleSchema(data=plist)

    @property
    async def all(self):
        data = await self.mc.all_info
        players_data = data["players"]
        players_model = Players(
            max_count=players_data["max"],
            online=players_data["online"],
            lst=players_data["list"],
        )
        return AllInfo(
            status=data["status"],
            name=data["name"],
            motd=data["motd"],
            players=players_model,
        )

    @property
    async def status(self):
        try:
            await self.mc.update()
            return BoolSchema(data=True)
        except ConnectionRefusedError:
            return BoolSchema(data=False)
